%%This is a part of multi-layered random forest (MLRF) framework

%Copyright (C) 2014 Alina Kuznetsova

%This program is free software: you can redistribute it and/or modify
%it under the terms of the GNU Lesser General Public License as published by
%the Free Software Foundation, either version 2.1 of the License, or
%(at your option) any later version.

%This program is distributed in the hope that it will be useful,
%but WITHOUT ANY WARRANTY; without even the implied warranty of
%MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%GNU Lesser General Public License for more details.

%You should have received a copy of the GNU General Public License
%along with this program.  If not, see <http://www.gnu.org/licenses/>.

%Author: Alina Kuznetsova

%% Compute initial RF-based clustering
%Input:
%   X - feature vectors
%   options - clustering options
%Output:
%   clusters - clustered data
%   sample2node - in which tree node an element ended up
%   maxclcount - maximal cluster cound
%   m - clustering forest
%   exclude - excluded feature coordinates
%   P - proximity matrix
%   link - hierarchical clustering link
function [clusters,sample2node,maxclcount,m,exclude,P,link] = createClustersRF(X,options)

    warning('Clustering the data');
    
    if(isfield(options,'aggLevel'))
        X = featureAggregation(X,options.aggLevel);
    end
    
    %%will it make difference???
    [X,Xmean,Xstd,exclude] = normalize(X,options);
    
    X = X; %scale?
    
    %may be scale X even more?
    
    disp([num2str(sum(exclude)) ' components excluded on clustering stage'])

    ids = randi(size(X,1),size(X));
    
    XtrainRand = X(ids);
    
    Xfull = [X; XtrainRand];
    Yfull = [ones(size(X,1),1); 2*ones(size(X,1),1)];
    
    [m,sample2node] = forestTrain(Xfull,Yfull,options);
    
    sample2node = sample2node(1:size(X,1),:);

    P = s2n2P(sample2node);
    
    P(sub2ind(size(P),1:size(P,1),1:size(P,1))) = 1;
    
    P = sqrt(1-P);
    
    %save('proximityMatrix.mat','P')
    
    Pv = zeros(1,size(P,1)*(size(P,1)-1)/2);
        
    for j=1:(size(P,1)-1)
        last = j-1;
        start = last*size(P,1)-last*(last+1)/2;
        intl = size(P,1)-j;          
        Pv(start+1:start+intl) = P(j+1:end,j);
    end
    
    link = linkage(Pv,'complete');
    
    [clusters,sizes,maxclcount] = optHieClustering(link,options);
    
    if (sum(sizes>options.small)<options.clnum)
        warning('Number of big-enough classes does not correspond to the requested number');
        disp(['Possible number of clusters: ' num2str(maxclcount)]);
        [clusters,sizes,~] = optHieClustering(link,maxclcount,options);
        if (sum(sizes>options.small)<maxclcount)
            save('/home/kuznetso/tmp/data.mat','link','maxclcount','options')
            error('Number of big-enough classes does not correspond to the requested number')
        end
    end

end